﻿using Buzz.MachineInterface;
using BuzzGUI.Common;
using System;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Interop;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace WDE.GuitarChordBox
{
    class ChordWindow : Window
    {
        private GuitarChordBoxMachine gcbm;
        private Grid chordsGrid;
        public NumericUpDown nudTranspose;
        private ScrollViewer scrollViewer;

        public ChordWindow(GuitarChordBoxMachine gcb, ResourceDictionary resources)
        {
            gcbm = gcb;
            
            Resources = resources;
            Style = TryFindResource("ThemeWindowStyle") as Style;
            
            this.WindowStyle = WindowStyle.ToolWindow;
            //this.ResizeMode = ResizeMode.NoResize;

            this.Width = 640;
            this.Height = 500;
            this.MinWidth = 640;
            this.MinHeight = 340;

            new WindowInteropHelper(this).Owner = BuzzGUI.Common.Global.MachineViewHwndSource.Handle;
            this.WindowStartupLocation = WindowStartupLocation.CenterOwner;

            Grid mainGrid = new Grid() { Margin = new Thickness(8, 0, 8, 0) };
            mainGrid.RowDefinitions.Add(new RowDefinition() { Height = new GridLength(50) });
            mainGrid.RowDefinitions.Add(new RowDefinition());

            Grid transposeGrid = new Grid() { Margin = new Thickness(8, 0, 6, 0) };
            transposeGrid.RowDefinitions.Add(new RowDefinition() { Height = new GridLength(50) });
            transposeGrid.ColumnDefinitions.Add(new ColumnDefinition() { Width = new GridLength(100) });
            transposeGrid.ColumnDefinitions.Add(new ColumnDefinition());
            transposeGrid.ColumnDefinitions.Add(new ColumnDefinition());
            Grid.SetRow(transposeGrid, 0);

            TextBlock tbTranspose = new TextBlock() { Text = "Transpose: ", HorizontalAlignment = HorizontalAlignment.Right, Margin = new Thickness(8, 16, 8, 0) };
            nudTranspose = new NumericUpDown()
            {
                Width = 80,
                Height = 20,
                DecimalPlaces = 0,
                Change = (decimal)1,
                Minimum = (decimal)-12,
                Maximum = (decimal)12,
                Margin = new Thickness(4, 4, 4, 4),
                Value = 0, // gcbm.Transpose - 12,
                HorizontalAlignment = HorizontalAlignment.Left
            };
            nudTranspose.ValueChanged += NudTranspose_ValueChanged;
            Grid.SetColumn(tbTranspose, 0);
            Grid.SetColumn(nudTranspose, 1);

            Button btStop = new Button()
            {
                Content = "◼",
                FontSize = 14,
                HorizontalAlignment = HorizontalAlignment.Right,
                Width = 60,
                Height = 30,
                Margin = new Thickness(1, 1, SystemParameters.VerticalScrollBarWidth, 1)
            };
            btStop.Click += BtStop_Click;
            Grid.SetColumn(btStop, 2);

            transposeGrid.Children.Add(tbTranspose);
            transposeGrid.Children.Add(nudTranspose);
            transposeGrid.Children.Add(btStop);

            mainGrid.Children.Add(transposeGrid);

            scrollViewer = new ScrollViewer() { VerticalAlignment = VerticalAlignment.Stretch, HorizontalAlignment = HorizontalAlignment.Stretch };


            UpdateChordsGrid();


            Grid.SetRow(scrollViewer, 1);

            mainGrid.Children.Add(scrollViewer);

            this.Content = mainGrid;
        }

        private void BtStop_Click(object sender, RoutedEventArgs e)
        {
            gcbm.TrackParamStringE(new Note(Note.Off), 0);
            gcbm.TrackParamStringA(new Note(Note.Off), 0);
            gcbm.TrackParamStringD(new Note(Note.Off), 0);
            gcbm.TrackParamStringG(new Note(Note.Off), 0);
            gcbm.TrackParamStringB(new Note(Note.Off), 0);
            gcbm.TrackParamStringE2(new Note(Note.Off), 0);
        }

        private void NudTranspose_ValueChanged(object sender, RoutedPropertyChangedEventArgs<decimal> e)
        {
            // gcbm.Transpose = (int)nudTranspose.Value + 12;
            UpdateChordsGrid();
        }

        private void UpdateChordsGrid()
        {
            scrollViewer.Content = null;

            chordsGrid = new Grid() { Margin = new Thickness(8, 0, 8, 0) };

            chordsGrid.ColumnDefinitions.Add(new ColumnDefinition()); // Index
            chordsGrid.ColumnDefinitions.Add(new ColumnDefinition()); // Chord
            chordsGrid.ColumnDefinitions.Add(new ColumnDefinition());
            chordsGrid.ColumnDefinitions.Add(new ColumnDefinition());
            chordsGrid.ColumnDefinitions.Add(new ColumnDefinition());
            chordsGrid.ColumnDefinitions.Add(new ColumnDefinition());
            chordsGrid.ColumnDefinitions.Add(new ColumnDefinition());
            chordsGrid.ColumnDefinitions.Add(new ColumnDefinition());
            chordsGrid.ColumnDefinitions.Add(new ColumnDefinition()); // Play
            // chordsGrid.ColumnDefinitions.Add(new ColumnDefinition()); // Copy

            for (int i = 0; i < GuitarNotes.Chords.Length; i++)
            {
                chordsGrid.RowDefinitions.Add(new RowDefinition() { Height = new GridLength(30) });

                TextBlock tbIndex = new TextBlock() { Text = i.ToString("X"), FontSize = 16 };
                tbIndex.MouseEnter += TbChord_MouseEnter;
                tbIndex.MouseLeave += TbChord_MouseLeave;
                Grid.SetColumn(tbIndex, 0);
                Grid.SetRow(tbIndex, i);
                chordsGrid.Children.Add(tbIndex);

                string[] chord = GuitarNotes.Chords[i].Split(',');
                TextBlock tbChord = new TextBlock() { Text = chord[0], FontWeight = FontWeights.Bold, FontSize = 16 };
                tbChord.MouseEnter += TbChord_MouseEnter;
                tbChord.MouseLeave += TbChord_MouseLeave;
                Grid.SetColumn(tbChord, 1);
                Grid.SetRow(tbChord, i);
                chordsGrid.Children.Add(tbChord);

                for (int j = 1; j < chord.Length; j++)
                {
                    string txt = chord[j];

                    if (txt == "-")
                    {
                        txt = "Off";
                    }
                    else
                    {
                        int iNote = BuzzNote.Parse(chord[j]);
                        int iMidiNote = BuzzNote.ToMIDINote(iNote);
                        iMidiNote += (int)nudTranspose.Value;
                        iNote = BuzzNote.FromMIDINote(iMidiNote);
                        txt = BuzzNote.TryToString(iNote);
                    }
                    TextBlock tbNote = new TextBlock { Text = txt, FontSize = 16 };
                    tbNote.MouseEnter += TbChord_MouseEnter;
                    tbNote.MouseLeave += TbChord_MouseLeave;
                    Grid.SetColumn(tbNote, j + 1);
                    Grid.SetRow(tbNote, i);
                    chordsGrid.Children.Add(tbNote);                    
                }

                Button btPlay = new Button()
                {
                    Content = "▶",
                    Tag = i,
                    FontSize = 14,
                    HorizontalAlignment = HorizontalAlignment.Right,
                    Width = 60,
                    Height = 28,
                    Margin = new Thickness(1)
                };
                btPlay.Click += BtPlay_Click;
                Grid.SetColumn(btPlay, 8);
                Grid.SetRow(btPlay, i);
                chordsGrid.Children.Add(btPlay);

                Button btCopy = new Button() { Content = "Copy", Tag = i, FontSize = 14 };
                btCopy.Click += BtCopy_Click;
                Grid.SetColumn(btCopy, 8);
                Grid.SetRow(btCopy, i);
                // chordsGrid.Children.Add(btCopy);
            }
            scrollViewer.Content = chordsGrid;
        }

        private void TbChord_MouseLeave(object sender, System.Windows.Input.MouseEventArgs e)
        {
            TextBlock tb = (TextBlock)sender;

            var fadeInAnimation = new DoubleAnimation(0d, new TimeSpan(0, 0, 0, 0, 500));
            tb.Background = new SolidColorBrush(Colors.DarkSlateGray);
            tb.Background.Opacity = 1;
            tb.Background.BeginAnimation(SolidColorBrush.OpacityProperty, fadeInAnimation);
        }

        private void TbChord_MouseEnter(object sender, System.Windows.Input.MouseEventArgs e)
        {
            TextBlock tb = (TextBlock)sender;

            var fadeInAnimation = new DoubleAnimation(1d, new TimeSpan(0, 0, 0, 0, 500));
            tb.Background = new SolidColorBrush(Colors.DarkSlateGray);
            tb.Background.Opacity = 0;
            tb.Background.BeginAnimation(SolidColorBrush.OpacityProperty, fadeInAnimation);
        }

        private void BtCopy_Click(object sender, RoutedEventArgs e)
        {

        }

        private void BtPlay_Click(object sender, RoutedEventArgs e)
        {
            int index = (int)((Button)sender).Tag;
            string[] chords = GuitarNotes.Chords[index].Split(',');

            string strNote = chords[1];
            gcbm.TrackParamStringE(GuitarNotes.Transpose(strNote, (int)nudTranspose.Value), 0);

            strNote = chords[2];
            gcbm.TrackParamStringA(GuitarNotes.Transpose(strNote, (int)nudTranspose.Value), 0);

            strNote = chords[3];
            gcbm.TrackParamStringD(GuitarNotes.Transpose(strNote, (int)nudTranspose.Value), 0);

            strNote = chords[4];
            gcbm.TrackParamStringG(GuitarNotes.Transpose(strNote, (int)nudTranspose.Value), 0);

            strNote = chords[5];
            gcbm.TrackParamStringB(GuitarNotes.Transpose(strNote, (int)nudTranspose.Value), 0);

            strNote = chords[6];
            gcbm.TrackParamStringE2(GuitarNotes.Transpose(strNote, (int)nudTranspose.Value), 0);
        }
    }
}
